using System;
using System.ComponentModel.Composition;
using System.Linq;
using System.Windows;
using System.Windows.Data;
using System.Windows.Media;
using Ascon.Pilot.SDK.TaskSample.TaskEditView;
using Ascon.Pilot.Theme.ColorScheme;
using Ascon.Pilot.Theme.Controls;
using Ascon.Pilot.SDK.Menu;

namespace Ascon.Pilot.SDK.TaskSample
{
    [Export(typeof(IMenu<TasksViewContext>))]
    public class TaskSamplePlugin : IMenu<TasksViewContext>
    {
        private readonly ITaskModifier _taskModifier;
        private readonly IObjectsRepository _repository;
        private readonly IPilotDialogService _pilotDialogService;
        private readonly ITabServiceProvider _tabServiceProvider;

        private const string CREATE_NEW_TASK = "miCreateNewTask";
        private const string CREATE_NEW_AGREEMENT_TASK = "miCreateNewAgreementTask";
        private const string CREATE_NEW_REVIEW_TASK = "miCreateNewReviewTask";
        private const string SILENT_MENU_ITEM_NAME = "SilentTaskSamplePluginkMenuItem";
        private const string EDIT_WORKFLOW = "miEditWorkflow";

        private ITaskObject _taskToEdit;

        [ImportingConstructor]
        public TaskSamplePlugin(ITaskModifier taskModifier, IObjectsRepository repository, IPilotDialogService pilotDialogService, ITabServiceProvider tabServiceProvider)
        {
            _taskModifier = taskModifier;
            _repository = repository;
            _pilotDialogService = pilotDialogService;
            _tabServiceProvider = tabServiceProvider;

            var accentColor = (Color)ColorConverter.ConvertFromString(pilotDialogService.AccentColor);
            ColorScheme.Initialize(accentColor, pilotDialogService.Theme);
        }

        public void Build(IMenuBuilder builder, TasksViewContext context)
        {
            var tasks = context.SelectedTasks.ToList();
            var t = builder.ItemNames;
            if (tasks.Count == 0)
            {
                builder.ReplaceItem(CREATE_NEW_TASK);
                builder.ReplaceItem(CREATE_NEW_AGREEMENT_TASK);
                builder.ReplaceItem(CREATE_NEW_REVIEW_TASK);
                builder.AddItem(SILENT_MENU_ITEM_NAME, 0).WithHeader("Create new task from code");
            }

            if (tasks.Count == 1)
            {
                builder.ReplaceItem("miEditWorkflow");
                _taskToEdit = tasks.First();
            }
        }

        public void OnMenuItemClick(string name, TasksViewContext context)
        {
            var kind = TaskKind.Task;
            if (name == CREATE_NEW_AGREEMENT_TASK)
                kind = TaskKind.Agreement;
            if (name == CREATE_NEW_REVIEW_TASK)
                kind = TaskKind.Acquaintance;

            if (name == CREATE_NEW_REVIEW_TASK || name == CREATE_NEW_AGREEMENT_TASK || name == CREATE_NEW_TASK)
            {
                ShowNewTaskDialog(kind);
            }

            if (name == SILENT_MENU_ITEM_NAME)
            {
                _taskModifier.NewTask(
                    "New task from extension " + DateTime.Now.ToString("g"),
                    "TaskDescription",
                    new[] { _repository.GetCurrentPerson().MainPosition.Position }, DateTime.MaxValue);

                _taskModifier.Apply();
            }

            if (name == EDIT_WORKFLOW)
            {
                ShowTaskEditDialog(_taskToEdit.Kind);
            }
        }

        private void ShowNewTaskDialog(TaskKind kind)
        {
            var window = new PureWindow();
            TaskEditViewModel taskEditViewModel;
            var titleFormat = "New task - {0}";
            switch (kind)
            {
                case TaskKind.Task:
                    taskEditViewModel = new TaskEditViewModel(_taskModifier, _repository, _pilotDialogService, _tabServiceProvider);
                    break;
                case TaskKind.Agreement:
                    taskEditViewModel = new AgreementTaskEditViewModel(_taskModifier, _repository, _pilotDialogService, _tabServiceProvider);
                    titleFormat = "New agreement - {0}";
                    break;
                case TaskKind.Acquaintance:
                    taskEditViewModel = new ReviewTaskEditViewModel(_taskModifier, _repository, _pilotDialogService, _tabServiceProvider);
                    titleFormat = "New review - {0}";
                    break;
                default:
                    throw new ArgumentOutOfRangeException(nameof(kind), kind, null);
            }

            var taskEditView = new TaskEditView.TaskEditView {DataContext = taskEditViewModel};
            window.Content = taskEditView;
            window.SetBinding(Window.TitleProperty, new Binding("Title") {Source = taskEditViewModel, StringFormat = titleFormat});
            window.WindowStartupLocation = WindowStartupLocation.CenterScreen;
            window.Height = 600;
            window.Width = 800;
            window.Show();
        }

        private void ShowTaskEditDialog(TaskKind kind)
        {
            var titleFormat = "Edit task - {0}";
            TaskEditViewModel taskEditViewModel;
            switch (kind)
            {
                case TaskKind.Task:
                    taskEditViewModel = new ModifyTaskEditViewModel(_taskToEdit, _taskModifier, _repository, _pilotDialogService, _tabServiceProvider);
                    break;
                case TaskKind.Agreement:
                    taskEditViewModel = new ModifyAgreementTaskViewModel(_taskToEdit, _taskModifier, _repository, _pilotDialogService, _tabServiceProvider);
                    titleFormat = "Edit agreement task - {0}";
                    break;
                case TaskKind.Acquaintance:
                    taskEditViewModel = new ModifyReviewTaskEditViewModel(_taskToEdit, _taskModifier, _repository, _pilotDialogService, _tabServiceProvider);
                    titleFormat = "Edit review task - {0}";
                    break;
                default:
                    throw new ArgumentOutOfRangeException(nameof(kind), kind, null);
            }

            var taskEditView = new TaskEditView.TaskEditView {DataContext = taskEditViewModel};
            var window = new PureWindow {Content = taskEditView};
            window.SetBinding(Window.TitleProperty, new Binding("Title") {Source = taskEditViewModel, StringFormat = titleFormat});
            window.WindowStartupLocation = WindowStartupLocation.CenterScreen;
            window.Height = 600;
            window.Width = 800;
            window.Show();
        }
    }
}
