﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using Ascon.Pilot.SDK;

namespace ProjectCloneWizard.Data
{
    public class DataObject
    {
        public DataObject(IDataObject dataObject)
        {
            Id = dataObject.Id;
            ParentId = dataObject.ParentId;
            Title = dataObject.DisplayName;
            Icon = dataObject.Type.SvgIcon;
            Sort = dataObject.Type.Sort;
            Type = new DataType(dataObject.Type);
            Children = new ReadOnlyCollection<Guid>(dataObject.Children);
            Attributes = new Dictionary<string, object>(dataObject.Attributes);
            Access = new ReadOnlyCollection<AccessRecord>(dataObject.Access2.Select(a => new AccessRecord(a)).ToList());
            Files = new ReadOnlyCollection<DataFile>(dataObject.Files.Select(f => new DataFile(f)).ToList());
            IsSecret = dataObject.IsSecret;
        }

        public Guid Id { get; private set; }
        public Guid ParentId { get; private set; }
        public string Title { get; private set; }
        public byte[] Icon { get; private set; }
        public int Sort { get; private set; }
        public DataType Type { get; }
        public ReadOnlyCollection<Guid> Children { get; private set; }
        public IDictionary<string, object> Attributes { get; private set; }
        public ReadOnlyCollection<AccessRecord> Access { get; set; }
        public ReadOnlyCollection<DataFile> Files { get; private set; }
        public bool IsSecret { get; private set; }
    }

    public class DataType
    {
        public DataType(IType type)
        {
            Id = type.Id;
            IsMountable = type.IsMountable;
            Attributes = new ReadOnlyCollection<AttributeWrapper>(type.Attributes.Select(x => new AttributeWrapper(x)).ToList());
            DisplayAttributes = new ReadOnlyCollection<AttributeWrapper>(type.DisplayAttributes.Select(x => new AttributeWrapper(x)).ToList());
            Title = type.Title;
            Name = type.Name;
            IsService = type.IsService;
            Kind = type.Kind;
            Sort = type.Sort;
            Children = new ReadOnlyCollection<int>(type.Children);
            HasFiles = type.HasFiles;
            Icon = type.SvgIcon;
            IsProject = type.IsProject;
            IsDeleted = type.IsDeleted;
        }

        public int Id { get; private set; }
        public bool IsMountable { get; private set; }
        public ReadOnlyCollection<AttributeWrapper> Attributes { get; private set; }
        public ReadOnlyCollection<AttributeWrapper> DisplayAttributes { get; private set; }
        public string Title { get; private set; }
        public string Name { get; private set; }
        public bool IsService { get; private set; }
        public TypeKind Kind { get; private set; }
        public int Sort { get; private set; }
        public ReadOnlyCollection<int> Children { get; private set; }
        public bool HasFiles { get; private set; }
        public byte[] Icon { get; private set; }
        public bool IsProject { get; private set; }
        public bool IsEcmDocument => IsMountable && HasFiles;
        public bool IsDeleted { get; private set; }
    }

    public class AccessRecord
    {
        public AccessRecord(IAccessRecord accessRecord)
        {
            OrgUnitId = accessRecord.OrgUnitId;
            Access = new Access(accessRecord.Access);
            RecordOwner = accessRecord.RecordOwner;
            InheritanceSource = accessRecord.InheritanceSource;
        }
        public int OrgUnitId { get; }
        public Access Access { get; }
        public int RecordOwner { get; }
        public Guid InheritanceSource { get; }
    }
    public class Access
    {
        public Access(IAccess access)
        {
            if (access == null)
                return;

            AccessLevel = access.AccessLevel;
            ValidThrough = access.ValidThrough;
            IsInheritable = access.IsInheritable;
            IsInherited = access.IsInherited;
        }
        public AccessLevel AccessLevel { get; private set; }
        public DateTime ValidThrough { get; private set; }
        public bool IsInheritable { get; private set; }
        public bool IsInherited { get; private set; }
    }

    public class DataFile : MarshalByRefObject, IFile
    {
        public DataFile(IFile file)
        {
            if (file == null)
                return;

            Id = file.Id;
            Size = file.Size;
            Md5 = file.Md5;
            Name = file.Name;
            Modified = file.Modified;
            Created = file.Created;
            Accessed = file.Accessed;
            Signatures = new ReadOnlyCollection<ISignature>(file.Signatures.Select(s => new SignatureWrapper(s)).Cast<ISignature>().ToList());
        }

        public Guid Id { get; private set; }
        public long Size { get; private set; }
        public string Md5 { get; private set; }
        public string Name { get; private set; }
        public DateTime Modified { get; private set; }
        public DateTime Created { get; private set; }
        public DateTime Accessed { get; private set; }
        public ReadOnlyCollection<ISignature> Signatures { get; private set; }
    }

    public class SignatureWrapper : MarshalByRefObject, ISignature
    {
        public SignatureWrapper(ISignature signature)
        {
            if (signature == null)
                return;

            Id = signature.Id;
            DatabaseId = signature.DatabaseId;
            PositionId = signature.PositionId;
            Role = signature.Role;
            Sign = signature.Sign;
            RequestedSigner = signature.RequestedSigner;
        }

        public Guid Id { get; private set; }
        public Guid DatabaseId { get; private set; }
        public int PositionId { get; private set; }
        public string Role { get; private set; }
        public string Sign { get; private set; }
        public string RequestedSigner { get; private set; }
    }

    internal class OrganisationUnit
    {
        public OrganisationUnit(IOrganisationUnit organisationUnit)
        {
            if (organisationUnit == null)
                return;

            Id = organisationUnit.Id;
            Title = organisationUnit.Title;
            IsDeleted = organisationUnit.IsDeleted;
            Children = new ReadOnlyCollection<int>(organisationUnit.Children);
            IsPosition = organisationUnit.IsPosition;
        }

        public int Id { get; private set; }
        public string Title { get; private set; }
        public bool IsDeleted { get; private set; }
        public ReadOnlyCollection<int> Children { get; private set; }
        public bool IsPosition { get; private set; }
    }

    internal class Person
    {
        public Person(IPerson creator)
        {
            if (creator == null)
                return;

            Id = creator.Id;
            Login = creator.Login;
            DisplayName = creator.DisplayName;
            Positions = new ReadOnlyCollection<PositionWrapper>(creator.Positions.Select(p => new PositionWrapper(p)).ToList());
            MainPosition = new PositionWrapper(creator.MainPosition);
            Comment = creator.Comment;
            Sid = creator.Sid;
            IsDeleted = creator.IsDeleted;
            IsAdmin = creator.IsAdmin;
            ActualName = creator.ActualName;
        }

        public int Id { get; private set; }
        public string Login { get; private set; }
        public string DisplayName { get; private set; }
        public ReadOnlyCollection<PositionWrapper> Positions { get; private set; }
        public PositionWrapper MainPosition { get; private set; }
        public string Comment { get; private set; }
        public string Sid { get; private set; }
        public bool IsDeleted { get; private set; }
        public bool IsAdmin { get; private set; }
        public string ActualName { get; private set; }
    }

    internal class PositionWrapper
    {
        public PositionWrapper(IPosition position)
        {
            if (position == null)
                return;

            Order = position.Order;
            Position = position.Position;
        }

        public int Order { get; private set; }
        public int Position { get; private set; }
    }

    public class AttributeWrapper
    {
        public string Name { get; private set; }
        public string Title { get; private set; }
        public AttributeType Type { get; private set; }
        public bool IsObligatory { get; private set; }
        public int DisplaySortOrder { get; private set; }
        public bool ShowInObjectsExplorer { get; private set; }
        public bool IsService { get; private set; }
        public string Configuration { get; private set; }
        public int DisplayHeight { get; private set; }

        public AttributeWrapper(IAttribute attribute)
        {
            Name = attribute.Name;
            Title = attribute.Title;
            IsObligatory = attribute.IsObligatory;
            DisplaySortOrder = attribute.DisplaySortOrder;
            ShowInObjectsExplorer = attribute.ShowInObjectsExplorer;
            IsService = attribute.IsService;
            Configuration = attribute.Configuration;
            DisplayHeight = attribute.DisplayHeight;
            Type = attribute.Type;
        }
    }
}