﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using Ascon.Pilot.SDK;
using ProjectCloneWizard.Data;
using ProjectCloneWizard.ViewModels;

namespace ProjectCloneWizard.ObjectCardView
{
    class ObjectCardViewModel : PropertyChangedBase, IObjectCardAutoComplete, IDataErrorInfo
    {
        public event EventHandler IsValidInputChanged;

        private readonly IObjectsRepository _repository;
        private readonly IPilotDialogService _dialogService;
        private readonly IAttributeFormatParser _attributeFormatParser;
        private DataType _type;
        private readonly Dictionary<string, object> _serviceAttributes = new Dictionary<string, object>();
        private ObservableCollection<CardControlViewModel> _viewModelCollection;
        private bool _isValidInput;

        internal event EventHandler<EventArgs> ValuesChanged;

        public ObjectCardViewModel(IObjectsRepository repository, IPilotDialogService dialogService, IAttributeFormatParser attributeFormatParser)
        {
            _repository = repository;
            _dialogService = dialogService;
            _attributeFormatParser = attributeFormatParser;
            _viewModelCollection = new ObservableCollection<CardControlViewModel>();
        }

        public ObservableCollection<CardControlViewModel> ViewModelCollection
        {
            get { return _viewModelCollection; }
            set { _viewModelCollection = value; }
        }

        public Dictionary<string, DValue> Values
        {
            get
            {
                var values = new Dictionary<string, DValue>();
                foreach (var cardControlViewModel in _viewModelCollection.Where(x => x.Value != null))
                {
                    values[cardControlViewModel.Attribute.Name] = DValue.GetDValue(cardControlViewModel.Value);
                }
                foreach (var serviceAttribute in _serviceAttributes)
                {
                    values[serviceAttribute.Key] = DValue.GetDValue(serviceAttribute.Value);
                }
                return values;
            }
        }

        public bool IsValidInput
        {
            get { return _isValidInput; }
            set
            {
                _isValidInput = value;
                RaiseIsValidInputChanged();
                NotifyOfPropertyChanged(() => IsValidInput);
            }
        }

        private bool _isCardUpdated;
        public bool IsCardUpdated
        {
            get { return _isCardUpdated; }
            set
            {
                _isCardUpdated = value;
                InvalidateIsValidInput();
            }
        }

        public IAttributeFormatParser AttributeFormatParser { get { return _attributeFormatParser; } }

        public bool IsDataChanged
        {
            get { return _viewModelCollection.Any(x => x.IsDataChanged); }
        }

        public DataType Type
        {
            get { return _type; }
            set
            {
                _type = value;
                NotifyOfPropertyChanged(() => Type);
            }
        }

        public void CreateCard(DataType type)
        {
            Type = type;

            _viewModelCollection.Clear();

            if (type != null && type.Attributes.Count > 0)
            {
                var attributes = type.Attributes.OrderBy(u => u.DisplaySortOrder).Where(a => a.IsService == false).ToArray();
                for (int i = 0; i < attributes.Length; i++)
                {
                    var attribute = attributes[i];
                    var initValue = GetNewAttrValue(attribute);
                    var cardControlViewModel = new CardControlViewModel(attribute, type, initValue, false, _dialogService, _repository, this, _attributeFormatParser, false); //TODO editMode?

                    cardControlViewModel.PropertyChanged += (o, e) => InvalidateIsValidInput();
                    _viewModelCollection.Add(cardControlViewModel);
                }
            }

            InvalidateIsValidInput();
        }

        private void InvalidateIsValidInput()
        {
            var isDirty = _viewModelCollection.Any(viewModel => viewModel.Value != null);
            var hasValidationError = _viewModelCollection.Any(viewModel => viewModel.HasValidationError);
            var obligatoryFieldsAreNotEmpty = _viewModelCollection.All(viewModel => !viewModel.Attribute.IsObligatory || (viewModel.Value != null && !string.IsNullOrWhiteSpace(viewModel.Value.ToString())));
            IsValidInput = isDirty && !hasValidationError && obligatoryFieldsAreNotEmpty;
            NotifyOfPropertyChanged(() => Values);
        }

        private void RaiseIsValidInputChanged()
        {
            IsValidInputChanged?.Invoke(this, EventArgs.Empty);
        }

        public void Fill(string attributeName, object value)
        {
            var viewModel = _viewModelCollection.FirstOrDefault(x => x.Attribute.Name == attributeName);
            if (viewModel != null)
            {
                // Отображаемый в карточке атрибут (кроме справочников)
                if (string.IsNullOrEmpty(viewModel.Attribute.Configuration))
                    viewModel.Value = value;
            }
            else
            {
                // Неотображаемый в карточке атрибут
                if (_type.Attributes.Any(x => x.Name == attributeName))
                    _serviceAttributes[attributeName] = value;
            }
            OnValuesChanged();
        }

        protected virtual void OnValuesChanged()
        {
            ValuesChanged?.Invoke(this, EventArgs.Empty);
            NotifyOfPropertyChanged(() => Values);
        }

        private static object GetNewAttrValue(AttributeWrapper attribute)
        {
            if (attribute.Type == AttributeType.DateTime)
                return DateTime.Now;
            return null;
        }

        #region validation

        public string Error
        {
            get { throw new NotImplementedException(); }
        }

        public string this[string columnName]
        {
            get
            {
                CardControlViewModel model = _viewModelCollection.First(c => c.Attribute.Name == columnName);
                if (!model.HasValidationError)
                    return model.Error;

                return String.Empty;
            }
        }
        #endregion
    }
}
